# ClickHouse query tools

Exposes ClickHouse internals to parse and manipulate ClickHouse queries.

Currently made of 2 modules:
* clickhouse-toolset: Includes only parser related functionality, like extracting or replacing tables from a query. It should be minimal since we use it in the CLI too.
* clickhouse-toolset-extras: Includes everything else. Only used in our analytics server.

## Installing prebuilts

Both modules are available in pypi:

```
pip install clickhouse-toolset
pip install clickhouse-toolset-extras
```

If we don't have prebuilts for your platform the installation will fail.

## No prebuilts available

To simplify things, the main module source distribution includes only the python code so that installing it is possible,
but it will throw when trying to use it:

```
>>> from chtoolset import query as chquery
Traceback (most recent call last):
  File "<stdin>", line 1, in <module>
  File "/home/raul/.local/lib/python3.9/site-packages/chtoolset/__init__.py", line 1, in <module>
    from . import query
  File "/home/raul/.local/lib/python3.9/site-packages/chtoolset/query.py", line 1, in <module>
    from chtoolset._query import replace_tables, format, tables, table_if_is_simple_query
ModuleNotFoundError: No module named 'chtoolset._query'
```

If you see this in the analytics server that means that your platform isn't supported and needs a prebuilt. If you see
this in the CLI that means that we are not handling the exception as it should (using a remote server).

## Development

First, you need to clone the repo and **its submodules**.

```
git clone --recursive git@gitlab.com:tinybird/clickhouse-toolset.git
```

Then, you will compile the dependencies and the module itself. You need a modern clang (currently Clang 12) to built it, both under Linux and OSX (AppleClang isn't supported either).

Since you can't have multiple modules in the same directory, sharing code and so on, currently we use `setup.py` and `setup-extras.py` to built them, for example:

```bash
python setup.py build
```

Since pip only supports `setup.py`, this means that `pip install -e .` will only work for the main module, but not the extra. This could be improved in the future.

Another option is to use the Makefile targets which will use virtualenv to install dependencies, build the packages, install them too and run tests:

```bash
make test-3.7
```

### Generate pre-built packages

You need to install all the necessary python releases so they are available via virtualenv, then simply do:

```
make build
```

#### Linux packages

In order to improve compatibility for Linux packages you should use auditwheel to "repair" them  before uploading them to pypi:

```
auditwheel repair dist/clickhouse_toolset-0.10.dev0-cp38-cp38-linux_x86_64.whl
```

This will check and rename them to manylinux1 (provided they have been compiled correctly).

## Examples

Check tests directory

## Publish

1. Update VERSION in `setup.py`

2. Publish the source package for the version you want to use to the **test repository**:

```
twine upload --repository-url https://test.pypi.org/legacy/ dist/clickhouse-toolset-extras-0.12.dev0.tar.gz
```

3. Publish the whl packages (wheelhouse/ is generated by auditwheel):

```
twine upload --repository-url https://test.pypi.org/legacy/ wheelhouse
```

4. Once tested, repeat for the production repository (no repository url)


## OSX

You need to be able to compile ClickHouse for OSX so we follow [their guide](https://github.com/ClickHouse/ClickHouse/blob/master/docs/en/development/build-osx.md) to install the necessary packages:

* Install Homebrew
* Install Xcode and Command Line Tools
* Install the necessary tools (cmake ninja libtool gettext llvm@12 gcc)
* Make sure your local clang is pointing to the llvm installation and to the default from OSX / Xcode.
* Follow the normal build (`make build`)
